@@ -140,10 +140,16 @@
 (define valgrind
   '("/usr/bin/valgrind" --leak-check=full --error-exitcode=154))
 
+(unless installed?
+	(setenv "GNUPG_BUILDDIR" (getenv "objdir") #t))
+
 (define (gpg-conf . args)
   (gpg-conf' "" args))
 (define (gpg-conf' input args)
-  (let ((s (call-popen `(,(tool-hardcoded 'gpgconf) ,@args) input)))
+  (let ((s (call-popen `(,(tool-hardcoded 'gpgconf)
+			 ,@(if installed? '()
+			       (list '--build-prefix (getenv "objdir")))
+			 ,@args) input)))
     (map (lambda (line) (map percent-decode (string-split line #\:)))
 	 (string-split-newlines s))))
 (define :gc:c:name car)
@@ -180,13 +186,7 @@
      (gpg-conf' (string-append key ":16:")
 		`(--change-options ,component)))))
 
-
-(unless installed?
-	(setenv "GNUPG_BUILDDIR" (getenv "objdir") #t))
-(define gpg-components (apply gpg-conf
-			`(,@(if installed? '()
-				(list '--build-prefix (getenv "objdir")))
-			  --list-components)))
+(define gpg-components (apply gpg-conf '(--list-components)))
 
 (define (tool which)
   (case which
@@ -201,7 +201,8 @@
 
 (define have-opt-always-trust
   (catch #f
-	 (call-check `(,(tool 'gpg) --gpgconf-test --always-trust))
+	 (with-ephemeral-home-directory (lambda ())
+	   (call-check `(,(tool 'gpg) --gpgconf-test --always-trust)))
 	 #t))
 
 (define GPG `(,(tool 'gpg) --no-permission-warning
@@ -258,6 +259,8 @@
   (not (not (member x (force all-hash-algos)))))
 (define (have-cipher-algo? x)
   (not (not (member x (force all-cipher-algos)))))
+(define (have-compression-algo? x)
+  (not (not (member x (force all-compression-algos)))))
 
 (define (gpg-pipe args0 args1 errfd)
   (lambda (source sink)
@@ -278,19 +281,6 @@
 ;; GnuPG helper.
 ;;
 
-;; Evaluate a sequence of expressions with an ephemeral home
-;; directory.
-(define-macro (with-ephemeral-home-directory . expressions)
-  (let ((original-home-directory (gensym))
-	(ephemeral-home-directory (gensym)))
-    `(let ((,original-home-directory (getenv "GNUPGHOME"))
-	   (,ephemeral-home-directory (mkdtemp)))
-       (finally (unlink-recursively ,ephemeral-home-directory)
-	 (dynamic-wind
-	     (lambda () (setenv "GNUPGHOME" ,ephemeral-home-directory #t))
-	     (lambda () ,@expressions)
-	     (lambda () (setenv "GNUPGHOME" ,original-home-directory #t)))))))
-
 ;; Call GPG to obtain the hash sums.  Either specify an input file in
 ;; ARGS, or an string in INPUT.  Returns a list of (<algo>
 ;; "<hashsum>") lists.
@@ -309,6 +299,12 @@
    (pipe:spawn `(,@GPG --dearmor))
    (pipe:write-to sink-name (logior O_WRONLY O_CREAT O_BINARY) #o600)))
 
+(define (gpg-dump-packets source-name sink-name)
+  (pipe:do
+   (pipe:open source-name (logior O_RDONLY O_BINARY))
+   (pipe:spawn `(,@GPG --list-packets))
+   (pipe:write-to sink-name (logior O_WRONLY O_CREAT O_BINARY) #o600)))
+
 ;;
 ;; Support for test environment creation and teardown.
 ;;
@@ -319,6 +315,13 @@
    (lambda (port)
      (display (make-random-string size) port))))
 
+(define (create-file name . lines)
+  (catch #f (unlink name))
+  (letfd ((fd (open name (logior O_WRONLY O_CREAT O_BINARY) #o600)))
+    (let ((port (fdopen fd "wb")))
+      (for-each (lambda (line) (display line port) (newline port))
+		lines))))
+
 (define (create-gpghome)
   (log "Creating test environment...")
 
@@ -326,21 +329,31 @@
   (make-test-data "random_seed" 600)
 
   (log "Creating configuration files")
-  (for-each
-   (lambda (name)
-     (file-copy (in-srcdir (string-append name ".tmpl")) name)
-     (let ((p (open-input-output-file name)))
-       (cond
-	((string=? "gpg.conf" name)
-	 (if have-opt-always-trust
-	     (display "no-auto-check-trustdb\n" p))
-	 (display (string-append "agent-program "
-				 (tool 'gpg-agent)
-				 "|--debug-quick-random\n") p)
-	 (display "allow-weak-digest-algos\n" p))
-	((string=? "gpg-agent.conf" name)
-	 (display (string-append "pinentry-program " PINENTRY "\n") p)))))
-   '("gpg.conf" "gpg-agent.conf")))
+
+  (if (flag "--use-keyring" *args*)
+      (create-file "pubring.gpg"))
+
+  (create-file "gpg.conf"
+	       "no-greeting"
+	       "no-secmem-warning"
+	       "no-permission-warning"
+	       "batch"
+	       "allow-weak-digest-algos"
+	       (if have-opt-always-trust
+		   "no-auto-check-trustdb" "#no-auto-check-trustdb")
+	       (string-append "agent-program "
+			      (tool 'gpg-agent)
+			      "|--debug-quick-random\n")
+	       )
+  (create-file "gpg-agent.conf"
+	       "allow-preset-passphrase"
+	       "no-grab"
+	       "enable-ssh-support"
+	       (if (flag "--extended-key-format" *args*)
+		   "enable-extended-key-format" "#enable-extended-key-format")
+	       (string-append "pinentry-program " (tool 'pinentry))
+	       (string-append "scdaemon-program " (tool 'scdaemon))
+	       ))
 
 ;; Initialize the test environment, install appropriate configuration
 ;; and start the agent, without any keys.
@@ -359,17 +372,16 @@
   (log "Unpacking samples")
   (for-each
    (lambda (name)
-     (dearmor (in-srcdir ".." "openpgp" (string-append name "o.asc")) name))
+     (dearmor (in-srcdir "tests" "openpgp" (string-append name "o.asc")) name))
    plain-files))
 
 (define (create-legacy-gpghome)
   (create-sample-files)
-  (mkdir "private-keys-v1.d" "-rwx")
 
   (log "Storing private keys")
   (for-each
    (lambda (name)
-     (dearmor (in-srcdir (string-append "/privkeys/" name ".asc"))
+     (dearmor (in-srcdir "tests" "openpgp" "privkeys" (string-append name ".asc"))
 	      (string-append "private-keys-v1.d/" name ".key")))
    '("50B2D4FA4122C212611048BC5FC31BD44393626E"
      "7E201E28B6FEB2927B321F443205F4724EBE637E"
@@ -393,11 +405,11 @@
   (log "Importing public demo and test keys")
   (for-each
    (lambda (file)
-     (call-check `(,@GPG --yes --import ,(in-srcdir file))))
+     (call-check `(,@GPG --yes --import ,(in-srcdir "tests" "openpgp" file))))
    (list "pubdemo.asc" "pubring.asc" key-file1))
 
   (pipe:do
-   (pipe:open (in-srcdir "pubring.pkr.asc") (logior O_RDONLY O_BINARY))
+   (pipe:open (in-srcdir "tests" "openpgp" "pubring.pkr.asc") (logior O_RDONLY O_BINARY))
    (pipe:spawn `(,@GPG --dearmor))
    (pipe:spawn `(,@GPG --yes --import))))
 
@@ -434,18 +446,23 @@
 ;; Create the socket dir and start the agent.
 (define (start-agent)
   (log "Starting gpg-agent...")
-  (atexit stop-agent)
+  (let ((gnupghome (getenv "GNUPGHOME")))
+    (atexit (lambda ()
+	      (with-home-directory gnupghome
+				   (stop-agent)))))
   (catch (log "Warning: Creating socket directory failed:" (car *error*))
-	 (call-popen `(,(tool 'gpgconf) --create-socketdir) ""))
+	 (gpg-conf '--create-socketdir))
   (call-check `(,(tool 'gpg-connect-agent) --verbose
 		,(string-append "--agent-program=" (tool 'gpg-agent)
 				"|--debug-quick-random")
 		/bye)))
 
-;; Stop the agent and remove the socket dir.
+;; Stop the agent and other daemons and remove the socket dir.
 (define (stop-agent)
   (log "Stopping gpg-agent...")
+  (gpg-conf '--kill 'all)
   (catch (log "Warning: Removing socket directory failed.")
-	 (call-popen `(,(tool 'gpgconf) --remove-socketdir) ""))
-  (call-check `(,(tool 'gpg-connect-agent) --verbose --no-autostart
-		killagent /bye)))
+	 (gpg-conf '--remove-socketdir)))
+
+
+;; end
